@@ -36,14 +36,25 @@
 
 #undef ENABLE_MEM_CHECK
 #undef ENABLE_MEM_PROFILE
+#undef ENABLE_GC_FRIENDLY
+
+#undef REALLOC_0_WORKS
 
 #undef G_COMPILED_WITH_DEBUGGING
+#undef G_ENABLE_MSG_PREFIX
 #undef G_THREADS_ENABLED
 
 #undef GLIB_SIZEOF_GMUTEX
+#undef GLIB_SIZEOF_INTMAX_T
+#undef GLIB_SIZEOF_PTRDIFF_T
+#undef GLIB_SIZEOF_SIZE_T
+#undef GLIB_SIZEOF_SYSTEM_THREAD
+
 #undef GLIB_BYTE_CONTENTS_GMUTEX
+#undef GLIB_BYTE_CONTENTS_GRECMUTEX
 
 #undef HAVE_BROKEN_WCTYPE
+#undef HAVE_CODESET
 #undef HAVE_DOPRNT
 #undef HAVE_FLOAT_H
 #undef HAVE_GETPWUID_R
@@ -51,8 +62,9 @@
 #undef HAVE_LIMITS_H
 #undef HAVE_LONG_DOUBLE
 #undef HAVE_POLL
-#undef HAVE_PTHREAD_GETSPECIFIC_POSIX
+#undef HAVE_PTHREAD_ATTR_SETSTACKSIZE
 #undef HAVE_PWD_H
+#undef HAVE_PW_GECOS
 #undef HAVE_SYS_PARAM_H
 #undef HAVE_SYS_POLL_H
 #undef HAVE_SYS_SELECT_H
@@ -64,14 +76,13 @@
 #undef HAVE_VALUES_H
 #undef HAVE_WCHAR_H
 #undef HAVE_WCTYPE_H
+#undef HAVE_WORKING_BCOPY
 
 #undef NO_FD_SET
 #undef NO_SYS_ERRLIST
 #undef NO_SYS_SIGLIST
 #undef NO_SYS_SIGLIST_DECL
 
-#undef WITH_SYMBOL_UNDERSCORE
-
 #undef SIZEOF_CHAR
 #undef SIZEOF_SHORT
 #undef SIZEOF_LONG
@@ -90,10 +101,10 @@
 #undef GLIB_INTERFACE_AGE
 #undef GLIB_BINARY_AGE
 
-#undef WIN32
-#undef NATIVE_WIN32
-
 #undef G_THREAD_SOURCE
+#undef POSIX_MIN_PRIORITY
+#undef POSIX_MAX_PRIORITY
+#undef POSIX_YIELD_FUNC
 
 /* #undef PACKAGE */
 /* #undef VERSION */
