@@ -17,6 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 /* 
  * MT safe
  */
@@ -27,7 +34,6 @@
 
 #define MIN_ARRAY_SIZE  16
 
-
 typedef struct _GRealArray  GRealArray;
 
 struct _GRealArray
@@ -40,6 +46,14 @@ struct _GRealArray
   guint   clear : 1;
 };
 
+#define g_array_elt_len(array,i) ((array)->elt_size * (i))
+#define g_array_elt_pos(array,i) ((array)->data + g_array_elt_len((array),(i)))
+#define g_array_elt_zero(array, pos, len) 				\
+  (memset (g_array_elt_pos ((array), pos), 0,  g_array_elt_len ((array), len)))
+#define g_array_zero_terminate(array) G_STMT_START{			\
+  if ((array)->zero_terminated)						\
+    g_array_elt_zero ((array), (array)->len, 1);			\
+}G_STMT_END
 
 static gint g_nearest_pow        (gint        num);
 static void g_array_maybe_expand (GRealArray *array,
@@ -52,6 +66,14 @@ GArray*
 g_array_new (gboolean zero_terminated,
 	     gboolean clear,
 	     guint    elt_size)
+{
+  return (GArray*) g_array_sized_new (zero_terminated, clear, elt_size, 0);
+}
+
+GArray* g_array_sized_new (gboolean zero_terminated,
+			   gboolean clear,
+			   guint    elt_size,
+			   guint    reserved_size)
 {
   GRealArray *array;
 
@@ -71,6 +93,12 @@ g_array_new (gboolean zero_terminated,
   array->clear           = (clear ? 1 : 0);
   array->elt_size        = elt_size;
 
+  if (array->zero_terminated || reserved_size != 0)
+    {
+      g_array_maybe_expand (array, reserved_size);
+      g_array_zero_terminate(array);
+    }
+
   return (GArray*) array;
 }
 
@@ -95,10 +123,13 @@ g_array_append_vals (GArray       *farray,
 
   g_array_maybe_expand (array, len);
 
-  memcpy (array->data + array->elt_size * array->len, data, array->elt_size * len);
+  memcpy (g_array_elt_pos (array, array->len), data, 
+	  g_array_elt_len (array, len));
 
   array->len += len;
 
+  g_array_zero_terminate (array);
+
   return farray;
 }
 
@@ -111,12 +142,15 @@ g_array_prepend_vals (GArray        *farray,
 
   g_array_maybe_expand (array, len);
 
-  g_memmove (array->data + array->elt_size * len, array->data, array->elt_size * array->len);
+  g_memmove (g_array_elt_pos (array, len), g_array_elt_pos (array, 0), 
+	     g_array_elt_len (array, array->len));
 
-  memcpy (array->data, data, len * array->elt_size);
+  memcpy (g_array_elt_pos (array, 0), data, g_array_elt_len (array, len));
 
   array->len += len;
 
+  g_array_zero_terminate (array);
+
   return farray;
 }
 
@@ -130,14 +164,16 @@ g_array_insert_vals (GArray        *farray,
 
   g_array_maybe_expand (array, len);
 
-  g_memmove (array->data + array->elt_size * (len + index), 
-	     array->data + array->elt_size * index, 
-	     array->elt_size * (array->len - index));
+  g_memmove (g_array_elt_pos (array, len + index), 
+	     g_array_elt_pos (array, index), 
+	     g_array_elt_len (array, array->len - index));
 
-  memcpy (array->data + array->elt_size * index, data, len * array->elt_size);
+  memcpy (g_array_elt_pos (array, index), data, g_array_elt_len (array, len));
 
   array->len += len;
 
+  g_array_zero_terminate (array);
+
   return farray;
 }
 
@@ -146,12 +182,22 @@ g_array_set_size (GArray *farray,
 		  guint   length)
 {
   GRealArray *array = (GRealArray*) farray;
-
-  if (array->len < length)
-    g_array_maybe_expand (array, length - array->len);
-
+  if (length > array->len)
+    {
+      g_array_maybe_expand (array, length - array->len);
+      
+      if (array->clear)
+	g_array_elt_zero (array, array->len, length - array->len);
+    }
+#ifdef ENABLE_GC_FRIENDLY  
+  else if (length < array->len)
+    g_array_elt_zero (array, length, array->len - length);
+#endif /* ENABLE_GC_FRIENDLY */  
+  
   array->len = length;
-
+  
+  g_array_zero_terminate (array);
+  
   return farray;
 }
 
@@ -163,43 +209,47 @@ g_array_remove_index (GArray* farray,
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index >= 0 && index < array->len, NULL);
+  g_return_val_if_fail (index < array->len, NULL);
 
   if (index != array->len - 1)
-      g_memmove (array->data + array->elt_size * index, 
-		 array->data + array->elt_size * (index + 1), 
-		 array->elt_size * (array->len - index - 1));
+    g_memmove (g_array_elt_pos (array, index),
+	       g_array_elt_pos (array, index + 1),
+	       g_array_elt_len (array, array->len - index - 1));
   
-  if (array->zero_terminated)
-    memset (array->data + array->elt_size * (array->len - 1), 0, 
-	    array->elt_size);
-
   array->len -= 1;
 
+#ifdef ENABLE_GC_FRIENDLY
+  g_array_elt_zero (array, array->len, 1);
+#else /* !ENABLE_GC_FRIENDLY */
+  g_array_zero_terminate (array);
+#endif /* ENABLE_GC_FRIENDLY */  
+
   return farray;
 }
 
 GArray*
 g_array_remove_index_fast (GArray* farray,
-			   guint index)
+			   guint   index)
 {
   GRealArray* array = (GRealArray*) farray;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index >= 0 && index < array->len, NULL);
+  g_return_val_if_fail (index < array->len, NULL);
 
   if (index != array->len - 1)
-    g_memmove (array->data + array->elt_size * index, 
-	       array->data + array->elt_size * (array->len - 1), 
-	       array->elt_size);
+    g_memmove (g_array_elt_pos (array, index), 
+	       g_array_elt_pos (array, array->len - 1),
+	       g_array_elt_len (array, 1));
   
-  if (array->zero_terminated)
-    memset (array->data + array->elt_size * (array->len - 1), 0, 
-	    array->elt_size);
-
   array->len -= 1;
 
+#ifdef ENABLE_GC_FRIENDLY
+  g_array_elt_zero (array, array->len, 1);
+#else /* !ENABLE_GC_FRIENDLY */
+  g_array_zero_terminate (array);
+#endif /* ENABLE_GC_FRIENDLY */  
+
   return farray;
 }
 
@@ -218,19 +268,21 @@ static void
 g_array_maybe_expand (GRealArray *array,
 		      gint        len)
 {
-  guint want_alloc = (array->len + len + array->zero_terminated) * array->elt_size;
+  guint want_alloc = g_array_elt_len (array, array->len + len + 
+				      array->zero_terminated);
 
   if (want_alloc > array->alloc)
     {
-      guint old_alloc = array->alloc;
+      want_alloc = g_nearest_pow (want_alloc);
+      want_alloc = MAX (want_alloc, MIN_ARRAY_SIZE);
 
-      array->alloc = g_nearest_pow (want_alloc);
-      array->alloc = MAX (array->alloc, MIN_ARRAY_SIZE);
+      array->data = g_realloc (array->data, want_alloc);
 
-      array->data = g_realloc (array->data, array->alloc);
+#ifdef ENABLE_GC_FRIENDLY
+      memset (array->data + array->alloc, 0, want_alloc - array->alloc);
+#endif /* ENABLE_GC_FRIENDLY */
 
-      if (array->clear || array->zero_terminated)
-	memset (array->data + old_alloc, 0, array->alloc - old_alloc);
+      array->alloc = want_alloc;
     }
 }
 
@@ -255,6 +307,12 @@ G_LOCK_DEFINE_STATIC (ptr_array_mem_chunk);
 
 GPtrArray*
 g_ptr_array_new (void)
+{
+  return g_ptr_array_sized_new (0);
+}
+
+GPtrArray*  
+g_ptr_array_sized_new (guint reserved_size)
 {
   GRealPtrArray *array;
 
@@ -271,7 +329,10 @@ g_ptr_array_new (void)
   array->len = 0;
   array->alloc = 0;
 
-  return (GPtrArray*) array;
+  if (reserved_size != 0)
+    g_ptr_array_maybe_expand (array, reserved_size);
+
+  return (GPtrArray*) array;  
 }
 
 void
@@ -292,20 +353,18 @@ static void
 g_ptr_array_maybe_expand (GRealPtrArray *array,
 			  gint        len)
 {
-  guint old_alloc;
-
   if ((array->len + len) > array->alloc)
     {
-      old_alloc = array->alloc;
-
+#ifdef ENABLE_GC_FRIENDLY
+      guint old_alloc = array->alloc;
+#endif /* ENABLE_GC_FRIENDLY */
       array->alloc = g_nearest_pow (array->len + len);
       array->alloc = MAX (array->alloc, MIN_ARRAY_SIZE);
-      if (array->pdata)
-	array->pdata = g_realloc (array->pdata, sizeof(gpointer) * array->alloc);
-      else
-	array->pdata = g_new0 (gpointer, array->alloc);
-
-      memset (array->pdata + old_alloc, 0, array->alloc - old_alloc);
+      array->pdata = g_realloc (array->pdata, sizeof(gpointer) * array->alloc);
+#ifdef ENABLE_GC_FRIENDLY
+      for ( ; old_alloc < array->alloc; old_alloc++)
+	array->pdata [old_alloc] = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
     }
 }
 
@@ -318,21 +377,40 @@ g_ptr_array_set_size  (GPtrArray   *farray,
   g_return_if_fail (array);
 
   if (length > array->len)
-    g_ptr_array_maybe_expand (array, (length - array->len));
+    {
+      int i;
+      g_ptr_array_maybe_expand (array, (length - array->len));
+      /* This is not 
+       *     memset (array->pdata + array->len, 0,
+       *            sizeof (gpointer) * (length - array->len));
+       * to make it really portable. Remember (void*)NULL needn't be
+       * bitwise zero. It of course is silly not to use memset (..,0,..).
+       */
+      for (i = array->len; i < length; i++)
+	array->pdata[i] = NULL;
+    }
+#ifdef ENABLE_GC_FRIENDLY  
+  else if (length < array->len)
+    {
+      int i;
+      for (i = length; i < array->len; i++)
+	array->pdata[i] = NULL;
+    }
+#endif /* ENABLE_GC_FRIENDLY */  
 
   array->len = length;
 }
 
 gpointer
 g_ptr_array_remove_index (GPtrArray* farray,
-			  guint index)
+			  guint      index)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   gpointer result;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index >= 0 && index < array->len, NULL);
+  g_return_val_if_fail (index < array->len, NULL);
 
   result = array->pdata[index];
   
@@ -340,33 +418,37 @@ g_ptr_array_remove_index (GPtrArray* farray,
     g_memmove (array->pdata + index, array->pdata + index + 1, 
 	       sizeof (gpointer) * (array->len - index - 1));
   
-  array->pdata[array->len - 1] = NULL;
-
   array->len -= 1;
 
+#ifdef ENABLE_GC_FRIENDLY  
+  array->pdata[array->len] = NULL;
+#endif /* ENABLE_GC_FRIENDLY */  
+
   return result;
 }
 
 gpointer
 g_ptr_array_remove_index_fast (GPtrArray* farray,
-			       guint index)
+			       guint      index)
 {
   GRealPtrArray* array = (GRealPtrArray*) farray;
   gpointer result;
 
   g_return_val_if_fail (array, NULL);
 
-  g_return_val_if_fail (index >= 0 && index < array->len, NULL);
+  g_return_val_if_fail (index < array->len, NULL);
 
   result = array->pdata[index];
   
   if (index != array->len - 1)
     array->pdata[index] = array->pdata[array->len - 1];
 
-  array->pdata[array->len - 1] = NULL;
-
   array->len -= 1;
 
+#ifdef ENABLE_GC_FRIENDLY  
+  array->pdata[array->len] = NULL;
+#endif /* ENABLE_GC_FRIENDLY */  
+
   return result;
 }
 
@@ -425,12 +507,17 @@ g_ptr_array_add (GPtrArray* farray,
   array->pdata[array->len++] = data;
 }
 
-/* Byte arrays
+/* Byte arrays 
  */
 
 GByteArray* g_byte_array_new      (void)
 {
-  return (GByteArray*) g_array_new (FALSE, FALSE, 1);
+  return (GByteArray*) g_array_sized_new (FALSE, FALSE, 1, 0);
+}
+
+GByteArray* g_byte_array_sized_new (guint reserved_size)
+{
+  return (GByteArray*) g_array_sized_new (FALSE, FALSE, 1, reserved_size);
 }
 
 void	    g_byte_array_free     (GByteArray *array,
