@@ -20,6 +20,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
+
 #include "config.h"
 #include "glib.h"
 #include <windows.h>
@@ -1018,10 +1026,10 @@ g_io_channel_win32_pipe_readable (gint  fd,
   for (i = 0; i < n_watched_pipes; i++)
     if (watched_pipes[i].fd == fd)
       {
-	if (watched_pipes[i].channel->offset < offset)
+	if ((watched_pipes[i].watch->condition & G_IO_IN)
+	    && watched_pipes[i].channel->offset < offset)
 	  (*watched_pipes[i].watch->callback) (watched_pipes[i].watch->channel,
 					       G_IO_IN,
 					       watched_pipes[i].user_data);
-	break;
       }
 }
