@@ -54,15 +54,14 @@
 #endif /* HAVE_UNISTD_H */
 #include <errno.h>
 
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
 #define STRICT
 #include <windows.h>
-#endif /* NATIVE_WIN32 */
+#endif /* G_OS_WIN32 */
 
-#ifdef _MSC_VER
-#include <fcntl.h>
-#include <io.h>
-#endif /* _MSC_VER */
+#ifdef G_OS_BEOS
+#include <net/socket.h>
+#endif /* G_OS_BEOS */
 
 /* Types */
 
@@ -169,13 +168,13 @@ static GMemChunk *poll_chunk;
 static guint n_poll_records = 0;
 
 #ifdef G_THREADS_ENABLED
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
 /* this pipe is used to wake up the main loop when a source is added.
  */
 static gint wake_up_pipe[2] = { -1, -1 };
-#else /* NATIVE_WIN32 */
+#else /* G_OS_WIN32 */
 static HANDLE wake_up_semaphore = NULL;
-#endif /* NATIVE_WIN32 */
+#endif /* G_OS_WIN32 */
 static GPollFD wake_up_rec;
 static gboolean poll_waiting = FALSE;
 
@@ -190,7 +189,7 @@ extern gint poll (GPollFD *ufds, guint nfsd, gint timeout);
 #  endif  /* !sun */
 static GPollFunc poll_func = (GPollFunc) poll;
 #else	/* !HAVE_POLL */
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
 
 static gint
 g_poll (GPollFD *fds, guint nfds, gint timeout)
@@ -328,7 +327,7 @@ g_poll (GPollFD *fds, guint nfds, gint timeout)
     return 1;
 }
 
-#else  /* !NATIVE_WIN32 */
+#else  /* !G_OS_WIN32 */
 
 /* The following implementation of poll() comes from the GNU C Library.
  * Copyright (C) 1994, 1996, 1997 Free Software Foundation, Inc.
@@ -338,7 +337,11 @@ g_poll (GPollFD *fds, guint nfds, gint timeout)
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
-#endif /* HAVE_SYS_SELECT_H_ */
+#endif /* HAVE_SYS_SELECT_H */
+
+#ifdef G_OS_BEOS
+#undef NO_FD_SET
+#endif /* G_OS_BEOS */
 
 #ifndef NO_FD_SET
 #  define SELECT_MASK fd_set
@@ -404,7 +407,7 @@ g_poll (GPollFD *fds,
   return ready;
 }
 
-#endif /* !NATIVE_WIN32 */
+#endif /* !G_OS_WIN32 */
 
 static GPollFunc poll_func = g_poll;
 #endif	/* !HAVE_POLL */
@@ -486,7 +489,7 @@ g_source_add (gint           priority,
   /* Now wake up the main loop if it is waiting in the poll() */
   g_main_wakeup ();
 #endif
-
+  
   G_UNLOCK (main_loop);
 
   return return_val;
@@ -588,7 +591,7 @@ g_source_remove_by_funcs_user_data (GSourceFuncs *funcs,
 void
 g_get_current_time (GTimeVal *result)
 {
-#ifndef _MSC_VER
+#ifndef G_OS_WIN32
   struct timeval r;
   g_return_if_fail (result != NULL);
 
@@ -723,7 +726,7 @@ g_main_iterate (gboolean block,
       G_UNLOCK (main_loop);
       return FALSE;
     }
-#endif
+#endif G_THREADS_ENABLED  
   
   /* If recursing, finish up current dispatch, before starting over */
   if (pending_dispatches)
@@ -974,7 +977,7 @@ g_main_poll (gint     timeout,
   gint npoll;
 
 #ifdef G_THREADS_ENABLED
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
   if (wake_up_pipe[0] < 0)
     {
       if (pipe (wake_up_pipe) < 0)
@@ -1031,7 +1034,9 @@ g_main_poll (gint     timeout,
 #endif
       
       G_UNLOCK (main_loop);
-      (*poll_func) (fd_array, npoll, timeout);
+      if ((*poll_func) (fd_array, npoll, timeout) < 0)
+	g_warning ("poll(2) failed due to: %s.",
+		   g_strerror (errno));
       G_LOCK (main_loop);
       
 #ifdef	G_MAIN_POLL_DEBUG
@@ -1074,7 +1079,7 @@ g_main_poll (gint     timeout,
 #ifdef G_THREADS_ENABLED
   if (!poll_waiting)
     {
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
       gchar c;
       read (wake_up_pipe[0], &c, 1);
 #endif
@@ -1181,6 +1186,10 @@ g_main_remove_poll (GPollFD *fd)
 	  else
 	    poll_records = pollrec->next;
 
+#ifdef ENABLE_GC_FRIENDLY
+	  pollrec->fd = NULL;  
+#endif /* ENABLE_GC_FRIENDLY */
+
 	  pollrec->next = poll_free_list;
 	  poll_free_list = pollrec;
 
@@ -1222,7 +1231,7 @@ g_main_wakeup (void)
   if (poll_waiting)
     {
       poll_waiting = FALSE;
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
       write (wake_up_pipe[1], "A", 1);
 #else
       ReleaseSemaphore (wake_up_semaphore, 1, NULL);
@@ -1249,33 +1258,33 @@ g_timeout_set_expiration (GTimeoutData *data,
     }
 }
 
-static gboolean 
-g_timeout_prepare  (gpointer  source_data, 
+static gboolean
+g_timeout_prepare  (gpointer  source_data,
 		    GTimeVal *current_time,
 		    gint     *timeout,
 		    gpointer  user_data)
 {
   glong msec;
   GTimeoutData *data = source_data;
-
-  msec = (data->expiration.tv_sec  - current_time->tv_sec) * 1000 +
-         (data->expiration.tv_usec - current_time->tv_usec) / 1000;
-
+  
+  msec = ((data->expiration.tv_sec  - current_time->tv_sec) * 1000 +
+	  (data->expiration.tv_usec - current_time->tv_usec) / 1000);
+  
   if (msec < 0)
     msec = 0;
   else if (msec > data->interval)
     {
-      /* The system time has been set backwards, so we 
+      /* The system time has been set backwards, so we
        * reset the expiration time to now + data->interval;
        * this at least avoids hanging for long periods of time.
        */
       g_timeout_set_expiration (data, current_time);
       msec = data->interval;
     }
-
+  
   *timeout = msec;
-
-  return (msec == 0);
+  
+  return msec == 0;
 }
 
 static gboolean 
@@ -1284,14 +1293,14 @@ g_timeout_check (gpointer  source_data,
 		 gpointer  user_data)
 {
   GTimeoutData *data = source_data;
-
-  return (data->expiration.tv_sec < current_time->tv_sec) ||
-         ((data->expiration.tv_sec == current_time->tv_sec) &&
-	  (data->expiration.tv_usec <= current_time->tv_usec));
+  
+  return ((data->expiration.tv_sec < current_time->tv_sec) ||
+	  ((data->expiration.tv_sec == current_time->tv_sec) &&
+	   (data->expiration.tv_usec <= current_time->tv_usec)));
 }
 
 static gboolean
-g_timeout_dispatch (gpointer source_data, 
+g_timeout_dispatch (gpointer source_data,
 		    GTimeVal *dispatch_time,
 		    gpointer user_data)
 {
@@ -1300,15 +1309,16 @@ g_timeout_dispatch (gpointer source_data,
   if (data->callback (user_data))
     {
       g_timeout_set_expiration (data, dispatch_time);
+
       return TRUE;
     }
   else
     return FALSE;
 }
 
-guint 
+guint
 g_timeout_add_full (gint           priority,
-		    guint          interval, 
+		    guint          interval,
 		    GSourceFunc    function,
 		    gpointer       data,
 		    GDestroyNotify notify)
@@ -1342,7 +1352,8 @@ g_idle_prepare  (gpointer  source_data,
 		 gint     *timeout,
 		 gpointer  user_data)
 {
-  timeout = 0;
+  *timeout = 0;
+
   return TRUE;
 }
 
