@@ -168,17 +168,6 @@ _g_module_build_path (const gchar *directory,
 }
 #endif	/* no implementation */
 
-#if defined (NATIVE_WIN32) && defined (__LCC__)
-int __stdcall 
-LibMain (void         *hinstDll,
-	 unsigned long dwReason,
-	 void         *reserved)
-{
-  return 1;
-}
-#endif /* NATIVE_WIN32 && __LCC__ */
-
-
 /* --- functions --- */
 gboolean
 g_module_supported (void)
@@ -382,7 +371,7 @@ g_module_symbol (GModule	*module,
   *symbol = _g_module_symbol (module->handle, symbol_name);
 #endif	/* !G_MODULE_NEED_USCORE */
   
-  module_error = g_module_error();
+  module_error = g_module_error ();
   if (module_error)
     {
       gchar *error;
@@ -391,6 +380,7 @@ g_module_symbol (GModule	*module,
       g_module_set_error (error);
       g_free (error);
       *symbol = NULL;
+
       return FALSE;
     }
   
