@@ -16,6 +16,14 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 #ifndef __GMODULE_H__
 #define __GMODULE_H__
 
@@ -32,11 +40,11 @@ extern const char      *g_log_domain_gmodule;
  * to feature Windows dll stubs.
  */
 #define	G_MODULE_IMPORT		extern
-#ifdef NATIVE_WIN32
+#if defined (G_OS_WIN32)
 #  define	G_MODULE_EXPORT		__declspec(dllexport)
-#else /* !NATIVE_WIN32 */
+#else /* !G_OS_WIN32 */
 #  define	G_MODULE_EXPORT
-#endif /* !NATIVE_WIN32 */
+#endif /* !G_OS_WIN32 */
 
 typedef enum
 {
