@@ -16,8 +16,17 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 #undef	G_LOG_DOMAIN
 #include	<gmodule.h>
+#include        "gmoduleconf.h"
 
 
 G_MODULE_EXPORT void
@@ -29,7 +38,7 @@ g_clash_func (void)
 typedef	void (*SimpleFunc) (void);
 typedef	void (*GModuleFunc) (GModule *);
 
-SimpleFunc gplugin_clash_func;
+static SimpleFunc plugin_clash_func = NULL;
 
 int
 main (int   arg,
@@ -44,13 +53,16 @@ main (int   arg,
   string = g_get_current_dir ();
   g_print ("testgmodule (%s):\n", string);
 
-#ifdef NATIVE_WIN32
+#if (G_MODULE_IMPL == G_MODULE_IMPL_WIN32)
   plugin_a = g_strconcat (string, "\\libgplugin_a.dll", NULL);
   plugin_b = g_strconcat (string, "\\libgplugin_b.dll", NULL);
-#else /* !NATIVE_WIN32 */
+#elif (G_MODULE_IMPL == G_MODULE_IMPL_DLD)
+  plugin_a = g_strconcat (string, "/.libs/", "libgplugin_a.sl", NULL);
+  plugin_b = g_strconcat (string, "/.libs/", "libgplugin_b.sl", NULL);
+#else /* neither DLD nor WIN32 */
   plugin_a = g_strconcat (string, "/.libs/", "libgplugin_a.so", NULL);
   plugin_b = g_strconcat (string, "/.libs/", "libgplugin_b.so", NULL);
-#endif /* NATIVE_WIN32 */
+#endif
   g_free (string);
 
   /* module handles
@@ -62,6 +74,15 @@ main (int   arg,
       g_print ("error: %s\n", g_module_error ());
       return 1;
     }
+  g_print ("check that not yet bound symbols in shared libraries of main module are retrievable:\n");
+  string = "g_module_close";
+  g_print ("retrive symbol `%s' from \"%s\":\n", string, g_basename (g_module_name (module_self)));
+  if (!g_module_symbol (module_self, string, (gpointer) &f_self))
+    {
+      g_print ("error: %s\n", g_module_error ());
+      return 1;
+    }
+  g_print ("retrived symbol `%s' as %p\n", string, f_self);
   g_print ("load plugin from \"%s\"\n", plugin_a);
   module_a = g_module_open (plugin_a, G_MODULE_BIND_LAZY);
   if (!module_a)
@@ -129,6 +150,10 @@ main (int   arg,
   /* get and call clashing plugin functions
    */
   string = "gplugin_clash_func";
+  g_print ("retrive symbol `%s' from \"%s\"\n", string, g_basename (g_module_name (module_self)));
+  if (!g_module_symbol (module_self, string, (gpointer) &f_self))
+    f_self = NULL;
+  g_print ("retrived function `%s' from self: %p\n", string, f_self);
   g_print ("retrive symbol `%s' from \"%s\"\n", string, g_basename (g_module_name (module_a)));
   if (!g_module_symbol (module_a, string, (gpointer) &f_a))
     {
@@ -142,11 +167,11 @@ main (int   arg,
       return 1;
     }
   g_print ("call plugin function(%p) A: ", f_a);
-  gplugin_clash_func = f_a;
-  gplugin_clash_func ();
+  plugin_clash_func = f_a;
+  plugin_clash_func ();
   g_print ("call plugin function(%p) B: ", f_b);
-  gplugin_clash_func = f_b;
-  gplugin_clash_func ();
+  plugin_clash_func = f_b;
+  plugin_clash_func ();
 
   /* call gmodule function form A
    */
