@@ -20,6 +20,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 /* 
  * MT safe
  */
@@ -75,7 +82,7 @@ void
 g_node_push_allocator (GAllocator *allocator)
 {
   G_LOCK (current_allocator);
-  g_node_validate_allocator ( allocator );
+  g_node_validate_allocator (allocator);
   allocator->last = current_allocator;
   current_allocator = allocator;
   G_UNLOCK (current_allocator);
@@ -141,6 +148,14 @@ g_nodes_free (GNode *node)
     {
       if (parent->children)
 	g_nodes_free (parent->children);
+
+#ifdef ENABLE_GC_FRIENDLY
+      parent->data = NULL;
+      parent->prev = NULL;
+      parent->parent = NULL;
+      parent->children = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+
       if (parent->next)
 	parent = parent->next;
       else
@@ -182,6 +197,24 @@ g_node_unlink (GNode *node)
   node->prev = NULL;
 }
 
+GNode*
+g_node_copy (GNode *node)
+{
+  GNode *new_node = NULL;
+  
+  if (node)
+    {
+      GNode *child;
+      
+      new_node = g_node_new (node->data);
+      
+      for (child = g_node_last_child (node); child; child = child->prev)
+	g_node_prepend (new_node, g_node_copy (child));
+    }
+  
+  return new_node;
+}
+
 GNode*
 g_node_insert (GNode *parent,
 	       gint   position,
@@ -910,6 +943,9 @@ g_node_first_sibling (GNode *node)
 {
   g_return_val_if_fail (node != NULL, NULL);
   
+  if (node->parent)
+    return node->parent->children;
+  
   while (node->prev)
     node = node->prev;
   
