@@ -48,7 +48,7 @@
 #include	<errno.h>
 #include	<sys/types.h>	/* needed for sys/stat.h */
 #include	<sys/stat.h>
-#ifdef _MSC_VER
+#ifdef G_OS_WIN32
 #include	<io.h>		/* For _read() */
 #endif
 
@@ -89,8 +89,9 @@ static GScannerConfig g_scanner_config_template =
    )			/* cset_identifier_first */,
   (
    G_CSET_a_2_z
-   "_0123456789"
+   "_"
    G_CSET_A_2_Z
+   G_CSET_DIGITS
    G_CSET_LATINS
    G_CSET_LATINC
    )			/* cset_identifier_nth */,
@@ -189,6 +190,8 @@ g_scanner_new (GScannerConfig	*config_templ)
   
   scanner->config->case_sensitive	= config_templ->case_sensitive;
   scanner->config->cset_skip_characters	= config_templ->cset_skip_characters;
+  if (!scanner->config->cset_skip_characters)
+    scanner->config->cset_skip_characters = "";
   scanner->config->cset_identifier_first= config_templ->cset_identifier_first;
   scanner->config->cset_identifier_nth	= config_templ->cset_identifier_nth;
   scanner->config->cpair_comment_single	= config_templ->cpair_comment_single;
@@ -1611,9 +1614,8 @@ g_scanner_get_token_ll	(GScanner	*scanner,
 	      token = G_TOKEN_COMMENT_SINGLE;
 	      in_comment_single = TRUE;
 	      gstring = g_string_new ("");
-	      while ((ch = g_scanner_get_char (scanner,
-					       line_p,
-					       position_p)) != 0)
+	      ch = g_scanner_get_char (scanner, line_p, position_p);
+	      while (ch != 0)
 		{
 		  if (ch == config->cpair_comment_single[1])
 		    {
@@ -1623,7 +1625,7 @@ g_scanner_get_token_ll	(GScanner	*scanner,
 		    }
 		  
 		  gstring = g_string_append_c (gstring, ch);
-		  ch = 0;
+		  ch = g_scanner_get_char (scanner, line_p, position_p);
 		}
 	    }
 	  else if (config->scan_identifier && ch &&
