@@ -28,6 +28,13 @@
  * MT safe
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,7 +42,6 @@
 #include <ctype.h>
 #include "glib.h"
 
-
 typedef struct _GRealStringChunk GRealStringChunk;
 typedef struct _GRealString      GRealString;
 
@@ -61,10 +67,14 @@ static GMemChunk *string_mem_chunk = NULL;
 /* Hash Functions.
  */
 
-gint
-g_str_equal (gconstpointer v, gconstpointer v2)
+gboolean
+g_str_equal (gconstpointer v1,
+	     gconstpointer v2)
 {
-  return strcmp ((const gchar*) v, (const gchar*)v2) == 0;
+  const gchar *string1 = v1;
+  const gchar *string2 = v2;
+  
+  return strcmp (string1, string2) == 0;
 }
 
 /* 31 bit hash function */
@@ -73,14 +83,15 @@ g_str_hash (gconstpointer key)
 {
   const char *p = key;
   guint h = *p;
-  
+
   if (h)
     for (p += 1; *p != '\0'; p++)
       h = (h << 5) - h + *p;
-  
+
   return h;
 }
 
+
 /* String Chunks.
  */
 
@@ -148,7 +159,7 @@ g_string_chunk_insert (GStringChunk *fchunk,
       chunk->storage_next = 0;
     }
 
-  pos = ((char*)chunk->storage_list->data) + chunk->storage_next;
+  pos = ((char *) chunk->storage_list->data) + chunk->storage_next;
 
   strcpy (pos, string);
 
@@ -182,8 +193,8 @@ g_string_chunk_insert_const (GStringChunk *fchunk,
 
 /* Strings.
  */
-static gint
-nearest_pow (gint num)
+static inline gint
+nearest_power (gint num)
 {
   gint n = 1;
 
@@ -198,7 +209,7 @@ g_string_maybe_expand (GRealString* string, gint len)
 {
   if (string->len + len >= string->alloc)
     {
-      string->alloc = nearest_pow (string->len + len + 1);
+      string->alloc = nearest_power (string->len + len + 1);
       string->str = g_realloc (string->str, string->alloc);
     }
 }
@@ -242,7 +253,7 @@ g_string_new (const gchar *init)
 
 void
 g_string_free (GString *string,
-	       gint free_segment)
+	       gboolean free_segment)
 {
   g_return_if_fail (string != NULL);
 
@@ -254,108 +265,170 @@ g_string_free (GString *string,
   G_UNLOCK (string_mem_chunk);
 }
 
-GString*
-g_string_assign (GString *lval,
-		 const gchar *rval)
+gboolean
+g_string_equal (const GString *v,
+                const GString *v2)
 {
-  g_string_truncate (lval, 0);
-  g_string_append (lval, rval);
+  gchar *p, *q;
+  GRealString *string1 = (GRealString *) v;
+  GRealString *string2 = (GRealString *) v2;
+  gint i = string1->len;
 
-  return lval;
+  if (i != string2->len)
+    return FALSE;
+
+  p = string1->str;
+  q = string2->str;
+  while (i)
+    {
+      if (*p != *q)
+	return FALSE;
+      p++;
+      q++;
+      i--;
+    }
+  return TRUE;
+}
+
+/* 31 bit hash function */
+guint
+g_string_hash (const GString *str)
+{
+  const gchar *p = str->str;
+  gint n = str->len;
+  guint h = 0;
+
+  while (n--)
+    {
+      h = (h << 5) - h + *p;
+      p++;
+    }
+
+  return h;
 }
 
 GString*
-g_string_truncate (GString* fstring,
-		   gint len)
+g_string_assign (GString     *string,
+		 const gchar *rval)
 {
-  GRealString *string = (GRealString*)fstring;
+  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (rval != NULL, string);
+  
+  g_string_truncate (string, 0);
+  g_string_append (string, rval);
+
+  return string;
+}
+
+GString*
+g_string_truncate (GString *fstring,
+		   guint    len)
+{
+  GRealString *string = (GRealString *) fstring;
 
   g_return_val_if_fail (string != NULL, NULL);
 
-  string->len = len;
+  string->len = MIN (len, string->len);
 
-  string->str[len] = 0;
+  string->str[string->len] = 0;
 
   return fstring;
 }
 
 GString*
-g_string_append (GString *fstring,
-		 const gchar *val)
+g_string_insert_len (GString     *fstring,
+		     gint         pos,
+		     const gchar *val,
+		     gint         len)
 {
-  GRealString *string = (GRealString*)fstring;
-  int len;
+  GRealString *string = (GRealString *) fstring;
 
   g_return_val_if_fail (string != NULL, NULL);
   g_return_val_if_fail (val != NULL, fstring);
+  g_return_val_if_fail (pos <= string->len, fstring);
+
+  if (len < 0)
+    len = strlen (val);
+
+  if (pos < 0)
+    pos = string->len;
   
-  len = strlen (val);
   g_string_maybe_expand (string, len);
 
-  strcpy (string->str + string->len, val);
+  /* If we aren't appending at the end, move a hunk
+   * of the old string to the end, opening up space
+   */
+  if (pos < string->len)
+    g_memmove (string->str + pos + len, string->str + pos, string->len - pos);
+  
+  /* insert the new string */
+  g_memmove (string->str + pos, val, len);
 
   string->len += len;
 
+  string->str[string->len] = 0;
+
   return fstring;
 }
 
+GString*
+g_string_append (GString     *fstring,
+		 const gchar *val)
+{  
+  g_return_val_if_fail (fstring != NULL, NULL);
+  g_return_val_if_fail (val != NULL, fstring);
+
+  return g_string_insert_len (fstring, -1, val, -1);
+}
+
+GString*
+g_string_append_len (GString	 *string,
+                     const gchar *val,
+                     gint         len)
+{
+  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (val != NULL, string);
+
+  return g_string_insert_len (string, -1, val, len);
+}
+
 GString*
 g_string_append_c (GString *fstring,
-		   gchar c)
+		   gchar    c)
 {
-  GRealString *string = (GRealString*)fstring;
+  g_return_val_if_fail (fstring != NULL, NULL);
 
-  g_return_val_if_fail (string != NULL, NULL);
-  g_string_maybe_expand (string, 1);
-
-  string->str[string->len++] = c;
-  string->str[string->len] = 0;
-
-  return fstring;
+  return g_string_insert_c (fstring, -1, c);
 }
 
 GString*
-g_string_prepend (GString *fstring,
+g_string_prepend (GString     *fstring,
 		  const gchar *val)
 {
-  GRealString *string = (GRealString*)fstring;
-  gint len;
-
-  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (fstring != NULL, NULL);
   g_return_val_if_fail (val != NULL, fstring);
+  
+  return g_string_insert_len (fstring, 0, val, -1);
+}
 
-  len = strlen (val);
-  g_string_maybe_expand (string, len);
+GString*
+g_string_prepend_len (GString	  *string,
+                      const gchar *val,
+                      gint         len)
+{
+  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (val != NULL, string);
 
-  g_memmove (string->str + len, string->str, string->len);
-
-  strncpy (string->str, val, len);
-
-  string->len += len;
-
-  string->str[string->len] = 0;
-
-  return fstring;
+  return g_string_insert_len (string, 0, val, len);
 }
 
 GString*
 g_string_prepend_c (GString *fstring,
 		    gchar    c)
-{
-  GRealString *string = (GRealString*)fstring;
-
-  g_return_val_if_fail (string != NULL, NULL);
-  g_string_maybe_expand (string, 1);
-
-  g_memmove (string->str + 1, string->str, string->len);
-
-  string->str[0] = c;
-
-  string->len += 1;
-
-  string->str[string->len] = 0;
-
-  return fstring;
+{  
+  g_return_val_if_fail (fstring != NULL, NULL);
+  
+  return g_string_insert_c (fstring, 0, c);
 }
 
 GString*
@@ -363,41 +436,31 @@ g_string_insert (GString     *fstring,
 		 gint         pos,
 		 const gchar *val)
 {
-  GRealString *string = (GRealString*)fstring;
-  gint len;
-
-  g_return_val_if_fail (string != NULL, NULL);
+  g_return_val_if_fail (fstring != NULL, NULL);
   g_return_val_if_fail (val != NULL, fstring);
-  g_return_val_if_fail (pos >= 0, fstring);
-  g_return_val_if_fail (pos <= string->len, fstring);
-
-  len = strlen (val);
-  g_string_maybe_expand (string, len);
-
-  g_memmove (string->str + pos + len, string->str + pos, string->len - pos);
-
-  strncpy (string->str + pos, val, len);
-
-  string->len += len;
-
-  string->str[string->len] = 0;
-
-  return fstring;
+  g_return_val_if_fail (pos <= fstring->len, fstring);
+  
+  return g_string_insert_len (fstring, pos, val, -1);
 }
 
-GString *
+GString*
 g_string_insert_c (GString *fstring,
 		   gint     pos,
 		   gchar    c)
 {
-  GRealString *string = (GRealString*)fstring;
+  GRealString *string = (GRealString *) fstring;
 
   g_return_val_if_fail (string != NULL, NULL);
   g_return_val_if_fail (pos <= string->len, fstring);
 
   g_string_maybe_expand (string, 1);
 
-  g_memmove (string->str + pos + 1, string->str + pos, string->len - pos);
+  if (pos < 0)
+    pos = string->len;
+  
+  /* If not just an append, move the old stuff */
+  if (pos < string->len)
+    g_memmove (string->str + pos + 1, string->str + pos, string->len - pos);
 
   string->str[pos] = c;
 
@@ -410,8 +473,8 @@ g_string_insert_c (GString *fstring,
 
 GString*
 g_string_erase (GString *fstring,
-		gint pos,
-		gint len)
+		gint     pos,
+		gint     len)
 {
   GRealString *string = (GRealString*)fstring;
 
@@ -434,17 +497,19 @@ g_string_erase (GString *fstring,
 GString*
 g_string_down (GString *fstring)
 {
-  GRealString *string = (GRealString*)fstring;
+  GRealString *string = (GRealString *) fstring;
   guchar *s;
+  gint n = string->len;
 
   g_return_val_if_fail (string != NULL, NULL);
 
   s = string->str;
 
-  while (*s)
+  while (n)
     {
       *s = tolower (*s);
       s++;
+      n--;
     }
 
   return fstring;
@@ -453,17 +518,19 @@ g_string_down (GString *fstring)
 GString*
 g_string_up (GString *fstring)
 {
-  GRealString *string = (GRealString*)fstring;
+  GRealString *string = (GRealString *) fstring;
   guchar *s;
+  gint n = string->len;
 
   g_return_val_if_fail (string != NULL, NULL);
 
   s = string->str;
 
-  while (*s)
+  while (n)
     {
       *s = toupper (*s);
       s++;
+      n--;
     }
 
   return fstring;
