@@ -17,6 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 /* 
  * MT safe
  */
@@ -53,7 +60,7 @@ static GTreeNode* g_tree_node_insert                (GTreeNode      *node,
 						     gint           *inserted);
 static GTreeNode* g_tree_node_remove                (GTreeNode      *node,
 						     GCompareFunc    compare,
-						     gpointer        key);
+						     gconstpointer   key);
 static GTreeNode* g_tree_node_balance               (GTreeNode      *node);
 static GTreeNode* g_tree_node_remove_leftmost       (GTreeNode      *node,
 						     GTreeNode     **leftmost);
@@ -63,7 +70,7 @@ static GTreeNode* g_tree_node_restore_right_balance (GTreeNode      *node,
 						     gint            old_balance);
 static gpointer   g_tree_node_lookup                (GTreeNode      *node,
 						     GCompareFunc    compare,
-						     gpointer        key);
+						     gconstpointer   key);
 static gint       g_tree_node_count                 (GTreeNode      *node);
 static gint       g_tree_node_pre_order             (GTreeNode      *node,
 						     GTraverseFunc   traverse_func,
@@ -75,8 +82,8 @@ static gint       g_tree_node_post_order            (GTreeNode      *node,
 						     GTraverseFunc   traverse_func,
 						     gpointer        data);
 static gpointer   g_tree_node_search                (GTreeNode      *node,
-						     GSearchFunc     search_func,
-						     gpointer        data);
+						     GCompareFunc    search_func,
+						     gconstpointer   data);
 static gint       g_tree_node_height                (GTreeNode      *node);
 static GTreeNode* g_tree_node_rotate_left           (GTreeNode      *node);
 static GTreeNode* g_tree_node_rotate_right          (GTreeNode      *node);
@@ -128,6 +135,13 @@ g_tree_node_destroy (GTreeNode *node)
     {
       g_tree_node_destroy (node->right);
       g_tree_node_destroy (node->left);
+
+#ifdef ENABLE_GC_FRIENDLY
+      node->left = NULL;
+      node->key = NULL;
+      node->value = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+
       G_LOCK (g_tree_global);
       node->right = node_free_list;
       node_free_list = node;
@@ -181,8 +195,8 @@ g_tree_insert (GTree    *tree,
 }
 
 void
-g_tree_remove (GTree    *tree,
-	       gpointer  key)
+g_tree_remove (GTree         *tree,
+	       gconstpointer  key)
 {
   GRealTree *rtree;
 
@@ -194,8 +208,8 @@ g_tree_remove (GTree    *tree,
 }
 
 gpointer
-g_tree_lookup (GTree    *tree,
-	       gpointer  key)
+g_tree_lookup (GTree         *tree,
+	       gconstpointer  key)
 {
   GRealTree *rtree;
 
@@ -242,9 +256,9 @@ g_tree_traverse (GTree         *tree,
 }
 
 gpointer
-g_tree_search (GTree       *tree,
-	       GSearchFunc  search_func,
-	       gpointer     data)
+g_tree_search (GTree            *tree,
+	       GCompareFunc      search_func,
+	       gconstpointer     data)
 {
   GRealTree *rtree;
 
@@ -357,9 +371,9 @@ g_tree_node_insert (GTreeNode    *node,
 }
 
 static GTreeNode*
-g_tree_node_remove (GTreeNode    *node,
-		    GCompareFunc  compare,
-		    gpointer      key)
+g_tree_node_remove (GTreeNode     *node,
+		    GCompareFunc   compare,
+		    gconstpointer  key)
 {
   GTreeNode *new_root;
   gint old_balance;
@@ -389,6 +403,12 @@ g_tree_node_remove (GTreeNode    *node,
 	  node = g_tree_node_restore_right_balance (new_root, old_balance);
 	}
 
+#ifdef ENABLE_GC_FRIENDLY
+      garbage->left = NULL;
+      garbage->key = NULL;
+      garbage->value = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+
       G_LOCK (g_tree_global);
       garbage->right = node_free_list;
       node_free_list = garbage;
@@ -483,9 +503,9 @@ g_tree_node_restore_right_balance (GTreeNode *node,
 }
 
 static gpointer
-g_tree_node_lookup (GTreeNode    *node,
-		    GCompareFunc  compare,
-		    gpointer      key)
+g_tree_node_lookup (GTreeNode     *node,
+		    GCompareFunc   compare,
+		    gconstpointer  key)
 {
   gint cmp;
 
@@ -588,9 +608,9 @@ g_tree_node_post_order (GTreeNode     *node,
 }
 
 static gpointer
-g_tree_node_search (GTreeNode   *node,
-		    GSearchFunc  search_func,
-		    gpointer     data)
+g_tree_node_search (GTreeNode     *node,
+		    GCompareFunc   search_func,
+		    gconstpointer  data)
 {
   gint dir;
 
@@ -637,12 +657,10 @@ g_tree_node_height (GTreeNode *node)
 static GTreeNode*
 g_tree_node_rotate_left (GTreeNode *node)
 {
-  GTreeNode *left;
   GTreeNode *right;
   gint a_bal;
   gint b_bal;
 
-  left = node->left;
   right = node->right;
 
   node->right = right->left;
@@ -675,12 +693,10 @@ static GTreeNode*
 g_tree_node_rotate_right (GTreeNode *node)
 {
   GTreeNode *left;
-  GTreeNode *right;
   gint a_bal;
   gint b_bal;
 
   left = node->left;
-  right = node->right;
 
   node->left = left->right;
   left->right = node;
