@@ -32,8 +32,6 @@
 #include <config.h>
 #endif
 
-#include "glibconfig.h"
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -50,17 +48,6 @@
 #include <sys/param.h>
 #endif
 
-#ifdef NATIVE_WIN32
-#  define STRICT			/* Strict typing, please */
-#  include <windows.h>
-#  include <direct.h>
-#  include <errno.h>
-#  include <ctype.h>
-#  ifdef _MSC_VER
-#    include <io.h>
-#  endif /* _MSC_VER */
-#endif /* NATIVE_WIN32 */
-
 /* implement Glib's inline functions
  */
 #define	G_INLINE_FUNC extern
@@ -68,12 +55,26 @@
 #include "glib.h"
 
 #ifdef	MAXPATHLEN
-#define	G_PATH_LENGTH	(MAXPATHLEN + 1)
+#define	G_PATH_LENGTH	MAXPATHLEN
 #elif	defined (PATH_MAX)
-#define	G_PATH_LENGTH	(PATH_MAX + 1)
-#else	/* !MAXPATHLEN */
-#define G_PATH_LENGTH   (2048 + 1)
-#endif	/* !MAXPATHLEN && !PATH_MAX */
+#define	G_PATH_LENGTH	PATH_MAX
+#elif   defined (_PC_PATH_MAX)
+#define	G_PATH_LENGTH	sysconf(_PC_PATH_MAX)
+#else	
+#define G_PATH_LENGTH   2048
+#endif
+
+#ifdef G_OS_WIN32
+#  define STRICT			/* Strict typing, please */
+#  include <windows.h>
+#  include <ctype.h>
+#  include <direct.h>
+#  include <io.h>
+#endif /* G_OS_WIN32 */
+
+#ifdef HAVE_CODESET
+#include <langinfo.h>
+#endif
 
 const guint glib_major_version = GLIB_MAJOR_VERSION;
 const guint glib_minor_version = GLIB_MINOR_VERSION;
@@ -81,15 +82,31 @@ const guint glib_micro_version = GLIB_MICRO_VERSION;
 const guint glib_interface_age = GLIB_INTERFACE_AGE;
 const guint glib_binary_age = GLIB_BINARY_AGE;
 
-#if defined (NATIVE_WIN32) && defined (__LCC__)
-int __stdcall 
-LibMain (void         *hinstDll,
-	 unsigned long dwReason,
-	 void         *reserved)
+#if !defined (HAVE_MEMMOVE) && !defined (HAVE_WORKING_BCOPY)
+void 
+g_memmove (gpointer dest, gconstpointer src, gulong len)
 {
-  return 1;
+  gchar* destptr = dest;
+  const gchar* srcptr = src;
+  if (src + len < dest || dest + len < src)
+    {
+      bcopy (src, dest, len);
+      return;
+    }
+  else if (dest <= src)
+    {
+      while (len--)
+	*(destptr++) = *(srcptr++);
+    }
+  else
+    {
+      destptr += len;
+      srcptr += len;
+      while (len--)
+	*(--destptr) = *(--srcptr);
+    }
 }
-#endif /* NATIVE_WIN32 && __LCC__ */
+#endif /* !HAVE_MEMMOVE && !HAVE_WORKING_BCOPY */
 
 void
 g_atexit (GVoidFunc func)
@@ -136,6 +153,10 @@ g_snprintf (gchar	*str,
   va_list args;
   gint retval;
   
+  g_return_val_if_fail (str != NULL, 0);
+  g_return_val_if_fail (n > 0, 0);
+  g_return_val_if_fail (fmt != NULL, 0);
+
   va_start (args, fmt);
   retval = vsnprintf (str, n, fmt, args);
   va_end (args);
@@ -151,6 +172,10 @@ g_snprintf (gchar	*str,
   gchar *printed;
   va_list args;
   
+  g_return_val_if_fail (str != NULL, 0);
+  g_return_val_if_fail (n > 0, 0);
+  g_return_val_if_fail (fmt != NULL, 0);
+
   va_start (args, fmt);
   printed = g_strdup_vprintf (fmt, args);
   va_end (args);
@@ -173,6 +198,10 @@ g_vsnprintf (gchar	 *str,
 #ifdef	HAVE_VSNPRINTF
   gint retval;
   
+  g_return_val_if_fail (str != NULL, 0);
+  g_return_val_if_fail (n > 0, 0);
+  g_return_val_if_fail (fmt != NULL, 0);
+
   retval = vsnprintf (str, n, fmt, args);
   
   if (retval < 0)
@@ -185,6 +214,10 @@ g_vsnprintf (gchar	 *str,
 #else	/* !HAVE_VSNPRINTF */
   gchar *printed;
   
+  g_return_val_if_fail (str != NULL, 0);
+  g_return_val_if_fail (n > 0, 0);
+  g_return_val_if_fail (fmt != NULL, 0);
+
   printed = g_strdup_vprintf (fmt, args);
   strncpy (str, printed, n);
   str[n-1] = '\0';
@@ -252,10 +285,10 @@ g_basename (const gchar	   *file_name)
   if (base)
     return base + 1;
 
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
   if (isalpha (file_name[0]) && file_name[1] == ':')
     return (gchar*) file_name + 2;
-#endif /* NATIVE_WIN32 */
+#endif /* G_OS_WIN32 */
   
   return (gchar*) file_name;
 }
@@ -268,7 +301,7 @@ g_path_is_absolute (const gchar *file_name)
   if (file_name[0] == G_DIR_SEPARATOR)
     return TRUE;
 
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
   if (isalpha (file_name[0]) && file_name[1] == ':' && file_name[2] == G_DIR_SEPARATOR)
     return TRUE;
 #endif
@@ -284,7 +317,7 @@ g_path_skip_root (gchar *file_name)
   if (file_name[0] == G_DIR_SEPARATOR)
     return file_name + 1;
 
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
   if (isalpha (file_name[0]) && file_name[1] == ':' && file_name[2] == G_DIR_SEPARATOR)
     return file_name + 3;
 #endif
@@ -317,19 +350,30 @@ g_dirname (const gchar	   *file_name)
 gchar*
 g_get_current_dir (void)
 {
-  gchar *buffer;
-  gchar *dir;
-
-  buffer = g_new (gchar, G_PATH_LENGTH);
-  *buffer = 0;
+  gchar *buffer = NULL;
+  gchar *dir = NULL;
+  static gulong max_len = (G_PATH_LENGTH == -1) ? 2048 : G_PATH_LENGTH;
   
   /* We don't use getcwd(3) on SUNOS, because, it does a popen("pwd")
    * and, if that wasn't bad enough, hangs in doing so.
    */
 #if	defined (sun) && !defined (__SVR4)
+  buffer = g_new (gchar, max_len + 1);
+  *buffer = 0;
   dir = getwd (buffer);
 #else	/* !sun */
-  dir = getcwd (buffer, G_PATH_LENGTH - 1);
+  while (max_len < G_MAXULONG / 2)
+    {
+      buffer = g_new (gchar, max_len + 1);
+      *buffer = 0;
+      dir = getcwd (buffer, max_len);
+
+      if (dir || errno != ERANGE)
+	break;
+
+      g_free (buffer);
+      max_len *= 2;
+    }
 #endif	/* !sun */
   
   if (!dir || !*buffer)
@@ -350,43 +394,76 @@ g_get_current_dir (void)
 gchar*
 g_getenv (const gchar *variable)
 {
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
   g_return_val_if_fail (variable != NULL, NULL);
 
   return getenv (variable);
 #else
-  gchar *v;
-  guint k;
-  static gchar *p = NULL;
-  static gint l;
+  G_LOCK_DEFINE_STATIC (getenv);
+  struct env_struct
+  {
+    gchar *key;
+    gchar *value;
+  } *env;
+  static GArray *environs = NULL;
+  gchar *system_env;
+  guint length, i;
   gchar dummy[2];
 
   g_return_val_if_fail (variable != NULL, NULL);
   
-  v = getenv (variable);
-  if (!v)
-    return NULL;
-  
+  G_LOCK (getenv);
+
+  if (!environs)
+    environs = g_array_new (FALSE, FALSE, sizeof (struct env_struct));
+
+  /* First we try to find the envinronment variable inside the already
+   * found ones.
+   */
+
+  for (i = 0; i < environs->len; i++)
+    {
+      env = &g_array_index (environs, struct env_struct, i);
+      if (strcmp (env->key, variable) == 0)
+	{
+	  g_assert (env->value);
+	  G_UNLOCK (getenv);
+	  return env->value;
+	}
+    }
+
+  /* If not found, we ask the system */
+
+  system_env = getenv (variable);
+  if (!system_env)
+    {
+      G_UNLOCK (getenv);
+      return NULL;
+    }
+
   /* On Windows NT, it is relatively typical that environment variables
    * contain references to other environment variables. Handle that by
    * calling ExpandEnvironmentStrings.
    */
 
+  g_array_set_size (environs, environs->len + 1);
+
+  env = &g_array_index (environs, struct env_struct, environs->len - 1);
+
   /* First check how much space we need */
-  k = ExpandEnvironmentStrings (v, dummy, 2);
-  /* Then allocate that much, and actualy do the expansion */
-  if (p == NULL)
-    {
-      p = g_malloc (k);
-      l = k;
-    }
-  else if (k > l)
-    {
-      p = g_realloc (p, k);
-      l = k;
-    }
-  ExpandEnvironmentStrings (v, p, k);
-  return p;
+  length = ExpandEnvironmentStrings (system_env, dummy, 2);
+
+  /* Then allocate that much, and actualy do the expansion and insert
+   * the new found pair into our buffer 
+   */
+
+  env->value = g_malloc (length);
+  env->key = g_strdup (variable);
+
+  ExpandEnvironmentStrings (system_env, env->value, length);
+
+  G_UNLOCK (getenv);
+  return env->value;
 #endif
 }
 
@@ -423,23 +500,23 @@ g_get_any_init (void)
       
       if (!g_tmp_dir)
 	{
-#ifndef NATIVE_WIN32
+#ifndef G_OS_WIN32
 	  g_tmp_dir = g_strdup ("/tmp");
-#else /* NATIVE_WIN32 */
+#else /* G_OS_WIN32 */
 	  g_tmp_dir = g_strdup ("C:\\");
-#endif /* NATIVE_WIN32 */
+#endif /* G_OS_WIN32 */
 	}
       
       if (!g_home_dir)
 	g_home_dir = g_strdup (g_getenv ("HOME"));
       
-#ifdef NATIVE_WIN32
+#ifdef G_OS_WIN32
       if (!g_home_dir)
 	{
 	  /* The official way to specify a home directory on NT is
 	   * the HOMEDRIVE and HOMEPATH environment variables.
 	   *
-	   * This is inside #ifdef NATIVE_WIN32 because with the cygwin dll,
+	   * This is inside #ifdef G_OS_WIN32 because with the cygwin dll,
 	   * HOME should be a POSIX style pathname.
 	   */
 	  
@@ -455,7 +532,7 @@ g_get_any_init (void)
 	      g_free (homepath);
 	    }
 	}
-#endif /* !NATIVE_WIN32 */
+#endif /* G_OS_WIN32 */
       
 #ifdef HAVE_PWD_H
       {
@@ -464,7 +541,12 @@ g_get_any_init (void)
 	
 #  ifdef HAVE_GETPWUID_R
         struct passwd pwd;
+#    ifdef _SC_GETPW_R_SIZE_MAX  
+	/* This reurns the maximum length */
+        guint bufsize = sysconf (_SC_GETPW_R_SIZE_MAX);
+#    else /* _SC_GETPW_R_SIZE_MAX */
         guint bufsize = 64;
+#    endif /* _SC_GETPW_R_SIZE_MAX */
         gint error;
 	
         do
@@ -495,8 +577,8 @@ g_get_any_init (void)
 		 */
 		if (error == 0 || error == ENOENT)
 		  {
-		    g_warning ("getpwuid_r(): failed due to: No such user %d.",
-			       getuid ());
+		    g_warning ("getpwuid_r(): failed due to unknown user id (%lu)",
+			       (gulong) getuid ());
 		    break;
 		  }
 		if (bufsize > 32 * 1024)
@@ -530,21 +612,25 @@ g_get_any_init (void)
       
 #else /* !HAVE_PWD_H */
       
-#  ifdef NATIVE_WIN32
+#  ifdef G_OS_WIN32
       {
 	guint len = 17;
 	gchar buffer[17];
 	
-	if (GetUserName (buffer, &len))
+	if (GetUserName ((LPTSTR) buffer, (LPDWORD) &len))
 	  {
 	    g_user_name = g_strdup (buffer);
 	    g_real_name = g_strdup (buffer);
 	  }
       }
-#  endif /* NATIVE_WIN32 */
+#  endif /* G_OS_WIN32 */
       
 #endif /* !HAVE_PWD_H */
       
+#ifdef __EMX__
+      /* change '\\' in %HOME% to '/' */
+      g_strdelimit (g_home_dir, "\\",'/');
+#endif
       if (!g_user_name)
 	g_user_name = g_strdup ("somebody");
       if (!g_real_name)
@@ -675,232 +761,30 @@ g_int_hash (gconstpointer v)
   return *(const gint*) v;
 }
 
-#if 0 /* Old IO Channels */
-
-GIOChannel*
-g_iochannel_new (gint fd)
+/**
+ * g_get_codeset:
+ * 
+ * Get the codeset for the current locale.
+ * 
+ * Return value: a newly allocated string containing the name
+ * of the codeset. This string must be freed with g_free().
+ **/
+gchar *
+g_get_codeset (void)
 {
-  GIOChannel *channel = g_new (GIOChannel, 1);
-
-  channel->fd = fd;
-
-#ifdef NATIVE_WIN32
-  channel->peer = 0;
-  channel->peer_fd = 0;
-  channel->offset = 0;
-  channel->need_wakeups = 0;
-#endif /* NATIVE_WIN32 */
-
-  return channel;
-}
-
-void
-g_iochannel_free (GIOChannel *channel)
-{
-  g_return_if_fail (channel != NULL);
-
-  g_free (channel);
-}
-
-void
-g_iochannel_close_and_free (GIOChannel *channel)
-{
-  g_return_if_fail (channel != NULL);
-
-  close (channel->fd);
-
-  g_iochannel_free (channel);
-}
-
-#undef g_iochannel_wakeup_peer
-
-void
-g_iochannel_wakeup_peer (GIOChannel *channel)
-{
-#ifdef NATIVE_WIN32
-  static guint message = 0;
+#ifdef HAVE_CODESET  
+  char *result = nl_langinfo (CODESET);
+  return g_strdup (result);
+#else
+#ifndef G_OS_WIN32
+  /* FIXME: Do something more intelligent based on setlocale (LC_CTYPE, NULL)
+   */
+  return g_strdup ("ISO-8859-1");
+#else
+  /* On Win32 we always use UTF-8. At least in GDK. SO should we
+   * therefore return that?
+   */
+  return g_strdup ("UTF-8");
+#endif
 #endif
-
-  g_return_if_fail (channel != NULL);
-
-#ifdef NATIVE_WIN32
-  if (message == 0)
-    message = RegisterWindowMessage ("gdk-pipe-readable");
-
-#  if 0
-  g_print ("g_iochannel_wakeup_peer: calling PostThreadMessage (%#x, %d, %d, %d)\n",
-	   channel->peer, message, channel->peer_fd, channel->offset);
-#  endif
-  PostThreadMessage (channel->peer, message,
-		     channel->peer_fd, channel->offset);
-#endif /* NATIVE_WIN32 */
 }
-
-#endif /* Old IO Channels */
-
-#ifdef NATIVE_WIN32
-#ifdef _MSC_VER
-
-int
-gwin_ftruncate (gint  fd,
-		guint size)
-{
-  HANDLE hfile;
-  guint curpos;
-
-  g_return_val_if_fail (fd >= 0, -1);
-  
-  hfile = (HANDLE) _get_osfhandle (fd);
-  curpos = SetFilePointer (hfile, 0, NULL, FILE_CURRENT);
-  if (curpos == 0xFFFFFFFF
-      || SetFilePointer (hfile, size, NULL, FILE_BEGIN) == 0xFFFFFFFF
-      || !SetEndOfFile (hfile))
-    {
-      gint error = GetLastError ();
-
-      switch (error)
-	{
-	case ERROR_INVALID_HANDLE:
-	  errno = EBADF;
-	  break;
-	default:
-	  errno = EIO;
-	  break;
-	}
-
-      return -1;
-    }
-
-  return 0;
-}
-
-DIR*
-gwin_opendir (const char *dirname)
-{
-  DIR *result;
-  gchar *mask;
-  guint k;
-
-  g_return_val_if_fail (dirname != NULL, NULL);
-
-  result = g_new0 (DIR, 1);
-  result->find_file_data = g_new0 (WIN32_FIND_DATA, 1);
-  result->dir_name = g_strdup (dirname);
-  
-  k = strlen (result->dir_name);
-  if (k && result->dir_name[k - 1] == '\\')
-    {
-      result->dir_name[k - 1] = '\0';
-      k--;
-    }
-  mask = g_strdup_printf ("%s\\*", result->dir_name);
-
-  result->find_file_handle = (guint) FindFirstFile (mask,
-					     (LPWIN32_FIND_DATA) result->find_file_data);
-  g_free (mask);
-
-  if (result->find_file_handle == (guint) INVALID_HANDLE_VALUE)
-    {
-      int error = GetLastError ();
-
-      g_free (result->dir_name);
-      g_free (result->find_file_data);
-      g_free (result);
-      switch (error)
-	{
-	default:
-	  errno = EIO;
-	  return NULL;
-	}
-    }
-  result->just_opened = TRUE;
-
-  return result;
-}
-
-struct dirent*
-gwin_readdir (DIR *dir)
-{
-  static struct dirent result;
-
-  g_return_val_if_fail (dir != NULL, NULL);
-
-  if (dir->just_opened)
-    dir->just_opened = FALSE;
-  else
-    {
-      if (!FindNextFile ((HANDLE) dir->find_file_handle,
-			 (LPWIN32_FIND_DATA) dir->find_file_data))
-	{
-	  int error = GetLastError ();
-
-	  switch (error)
-	    {
-	    case ERROR_NO_MORE_FILES:
-	      return NULL;
-	    default:
-	      errno = EIO;
-	      return NULL;
-	    }
-	}
-    }
-  strcpy (result.d_name, g_basename (((LPWIN32_FIND_DATA) dir->find_file_data)->cFileName));
-      
-  return &result;
-}
-
-void
-gwin_rewinddir (DIR *dir)
-{
-  gchar *mask;
-
-  g_return_if_fail (dir != NULL);
-
-  if (!FindClose ((HANDLE) dir->find_file_handle))
-    g_warning ("gwin_rewinddir(): FindClose() failed\n");
-
-  mask = g_strdup_printf ("%s\\*", dir->dir_name);
-  dir->find_file_handle = (guint) FindFirstFile (mask,
-					  (LPWIN32_FIND_DATA) dir->find_file_data);
-  g_free (mask);
-
-  if (dir->find_file_handle == (guint) INVALID_HANDLE_VALUE)
-    {
-      int error = GetLastError ();
-
-      switch (error)
-	{
-	default:
-	  errno = EIO;
-	  return;
-	}
-    }
-  dir->just_opened = TRUE;
-}  
-
-gint
-gwin_closedir (DIR *dir)
-{
-  g_return_val_if_fail (dir != NULL, -1);
-
-  if (!FindClose ((HANDLE) dir->find_file_handle))
-    {
-      int error = GetLastError ();
-
-      switch (error)
-	{
-	default:
-	  errno = EIO; return -1;
-	}
-    }
-
-  g_free (dir->dir_name);
-  g_free (dir->find_file_data);
-  g_free (dir);
-
-  return 0;
-}
-
-#endif /* _MSC_VER */
-
-#endif /* NATIVE_WIN32 */
