@@ -16,6 +16,14 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+/*
+ * Modified by the GLib Team and others 1997-1999.  See the AUTHORS
+ * file for a list of people on the GLib Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
+ */
+
 #undef G_LOG_DOMAIN
 
 #include <stdio.h>
@@ -91,6 +99,16 @@ main (int   argc,
   g_assert (strcmp(string, "00021 test ") == 0);
   g_free (string);
 
+  g_assert (strcmp
+	    (g_strcompress("abc\\\\\\\"\\b\\f\\n\\r\\t\\003\\177\\234\\313\\12345z"),
+	     "abc\\\"\b\f\n\r\t\003\177\234\313\12345z") == 0);
+  g_assert (strcmp(g_strescape("abc\\\"\b\f\n\r\t\003\177\234\313",
+			       NULL),
+		   "abc\\\\\\\"\\b\\f\\n\\r\\t\\003\\177\\234\\313") == 0);
+  g_assert (strcmp(g_strescape("abc\\\"\b\f\n\r\t\003\177\234\313",
+			       "\b\f\001\002\003\004"),
+		   "abc\\\\\\\"\b\f\\n\\r\\t\003\\177\\234\\313") == 0);
+
   return 0;
 }
 
